﻿#ifndef _POOL_h
#define _POOL_h

#include <algorithm> // 'std::any_of'
#include <array> 
#include <initializer_list>

template<typename T, unsigned capacity = 1000> class Pool {
private:
	std::array<T, capacity> _buffer; // Recycling occurs here...
public:
	typedef T CONTENT; // For convenience...
	bool allocate(const T& x) {
		return std::any_of(_buffer.begin(), _buffer.end(), [&x](T& t) -> bool {
			if (t.is_null()) {
				t = x;
				return true; // 'break'
			}
			else return false; // 'continue'
			});
	}
	bool desallocate(const T& x) {
		return std::any_of(_buffer.begin(), _buffer.end(), [&x](T& t) -> bool {
			if (t == x) {
				t.nullify();
				return true; // 'break'
			}
			else return false; // 'continue'
			});
	}
};

template<typename T = char> class Facade { // Pure functional object...
private:
	Pool<T>& _pool;
public: // Resource injection:
	Facade(Pool<T>& pool) : _pool(pool) {}
	// CRUD
	void Create(std::initializer_list<T>&& list) {
		for (auto& element : list)
			_pool.allocate(element);
	}
	// Read...
	// Update...
	void Delete(std::initializer_list<T>&& list) {
		for (auto& element : list)
			_pool.desallocate(element);
	}
};

#endif